/*
  A simple example of FORTRAN that call C++ code.
  The interface pass to routine with C-link interface
  in order that FORTRAN can call it.
  
  The interface read from a file the data organized 
  as columns with the first line a headers
  
  C1 C2      CN   <- header line
  V1 V2      V2   <- data line with numbers
  V1 V2      V2   <- data line with numbers
  V1 V2      V2   <- data line with numbers
  V1 V2      V2   <- data line with numbers
  ...
  
*/

#include <iostream>
#include <fstream>
#include <vector>
#include <string>
#include <sstream>

using namespace std ;

// normally fortran add _ (underscore) at the end of the name

#define F77NAME(A) A##_

//mappint of the type with fortran DOUBLE PRECISION and INTEGER
typedef double double_precision ;
typedef int    integer ;

/*
  Declaration of the C-interface, FORTRAN callable.
 */

extern "C" {

  // routine called to parse the file
  void
  F77NAME(parser_load_file)( char fileName[] ) ;

  // ask for the readed dimension (for dynamical allocation or checl).
  void
  F77NAME(parser_get_dimension)( integer & numRows,
                                 integer & numColumn ) ;
  void
  F77NAME(parser_get_data)( double_precision   A[],
                            integer          & ldA, // dimension of row of matrix A
                            char               S[],
                            integer          & ldS ) ;
}


/* memorize the read data to this vectors */

static vector<vector<double_precision> > a    ;
static vector<string>                    strs ;

extern "C"
void
F77NAME(parser_load_file)( char fileName[] ) {
  // step 1: open the file
  ifstream file(fileName) ;
  
  // some check ...

  // step 2: read the header
  string line, str ;
  getline( file, line ) ;
  
  // wrap the readed line to the object lineStream of type
  // istringstream. This object is a stream object the result
  // is that now the string can be read as a file...
  istringstream lineStream(line) ;
  
  strs . clear();
  while ( lineStream.good() ) {
    lineStream >> str ;
    if ( str . length() > 0 )
      strs . push_back(str) ;
  }

  // step 2: read the data
  a . resize(strs.size()) ;
  while ( file.good() ) {
    getline( file, line ) ;
    // same trick as for the header 
    istringstream lineStream(line) ;
    if ( line . length() > 0 ) { 
      for ( integer i = 0 ; i < strs.size() ; ++i ) {
        double_precision bf ;
        lineStream >> bf ;
        a[i] . push_back(bf) ;
      }
    }
  }
  file . close() ;
}

extern "C"
void
F77NAME(parser_get_dimension)( integer & numRows,
                               integer & numColumn ) {
  numRows   = a[0].size() ;
  numColumn = a.size() ;
}

extern "C"
void
F77NAME(parser_get_data)( double_precision   A[],
                          integer          & ldA, // dimension of row of matrix A
                          char               S[],
                          integer          & ldS ) {

  integer numRows   = a[0].size() ;
  integer numColumn = a.size() ;
  
  for ( integer j = 0 ; j < numColumn ; ++j )
    for ( integer i = 0 ; i < numRows ; ++i )
      A[i+j*ldA] = a[j][i] ; // j+i*ldA addess of the (i,j) in FORTRAN convection

  fill( S, S+numColumn*ldS, ' ' ) ;
  for ( integer j = 0 ; j < numColumn ; ++j ) {
    copy( strs[j].begin(), strs[j].end(), S+j*ldS ) ;
    //for ( integer k = 0 ; k < 100 ; ++k )
    //  cout << k << " " << (int)S[j*ldS+k] << endl ;
  }
}
